'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var hexToChannels = (function (hex) {
  var channel = function channel(idx) {
    return parseInt(hex.substr(idx * 2 + 1, 2), 16);
  };

  return {
    R: channel(0),
    G: channel(1),
    B: channel(2)
  };
});

var CHANNEL_SPECIFIC_LUM = {
  R: 0.35,
  G: 0.5,
  B: 0.15
};
/*
const IM_WONDERING_IF_THIS_IS_NOT_VERY_ACCURATE = {
  R: 0.2126,
  G: 0.7152,
  B: 0.0722,
}
*/

function funnel (value, _ref) {
  var min = _ref[0],
      max = _ref[1];
  return value > max ? max : value < min ? min : value;
}

var interpolate = (function (_ref) {
  var _ref$completionRatio = _ref.completionRatio,
      completionRatio = _ref$completionRatio === void 0 ? 0.5 : _ref$completionRatio,
      _ref$range = _ref.range,
      to = _ref$range[0],
      from = _ref$range[1];
  var value = from + completionRatio * (to - from);
  /*
  console.log('||| to', to, 'from', from)
  console.log("||| completionRatio", completionRatio)
  console.log("||| value", value)
  */

  return value;
});

function wrapAround(value, _ref) {
  var min = _ref[0],
      max = _ref[1];

  while (value >= max) {
    value -= max;
  }

  while (value < min) {
    value += max;
  }

  return value;
}

/**
 * Gives us the relative values of the channels,
 * irrespective of the white light beneath them.
 * @param {number} hue - in degrees. Gets safely wrapped around first thing.
 *
 * DgiRed    0 ->   0
 *
 * Sunlgt   50 ->  50
 *
 * DgiYlw   60 ->  60
 *
 * Citron   70 ->  70
 *
 * Turqse  510 -> 150
 *
 * @const {number} hueReduced - hue 0-359.9 is now a floating point 0-5.999.
 *
 * DgiRed    0 ->   0  ~  0.000
 *
 * Sunlgt   50 -> 5/6  ~  0.833
 *
 * DgiYlw   60 ->   1  ~  1.000
 *
 * Citron   70 -> 7/6  ~  1.167
 *
 * Turqse  150 -> 5/2  ~  2.500
 *
 * @const {number} hueInteger - from 1-6. Tells us what color region we are in.
 *
 * DgiRed  0.000 -> 0
 *                : red-into-yellow region
 *
 * Sunlgt  0.833 -> 0
 *                : red-into-yellow region
 *
 * DgiYlw  1.000 -> 1
 *                : yellow-into-green region
 *
 * Citron  1.167 -> 1
 *                : yellow-into-green region
 *
 * Turqse  2.500 -> 2
 *                : green-into-cyan region
 *
 * hueInteger is the 'whole number' piece of hueReduced.
 * FYI, the six color regions are bounded by red, yellow, green, cyan, blue, magenta.
 *
 * @const {number} hueDecimal - tells where we are in this region.
 *
 * DgiRed  0.000 -> 0.000
 *                : at the very beginning
 *
 * Sunlgt  0.833 -> 0.833
 *                : near the end
 *
 * DgiYlw  1.000 -> 0.000
 *                : at the very beginning
 *
 * Citron  1.167 -> 0.167
 *                : near the beginning
 *
 * Turqse  2.500 -> 0.500
 *                : at the halfway point
 *
 * hueDecimal is the 'fraction' piece of hueReduced.
 * we are going to use this number to determine the value of the in-between channel.
 *
 * @const {number} x - used in primary-secondary transitions like Red into Yellow
 * @const {number} y - used in secondary-primary transitions like Yellow into Green
   * To understand the function of x and y, take the difference between
   *
   * Sunlgt (hue 50),
   * Citron (hue 70),
   * DgiYlw (hue 60),
   *
   * as an instructive case. These colors are all basically yellow.
   *
   * Sunlgt is hue 50, which puts it near the end of the red-into-yellow region.
     *
     * This means its Red channel is full, and its Green channel is almost full.
     * The fullness of its Green channel is directly proportional to its
     * hueDecimal, the distance from the beginning of this region: 0.833
     *
   * Citron is hue 70, which puts it near the beginning of the yellow-into-green region.
     *
     * This means its Red channel is ALMOST FULL, and its Green channel is FULL.
     * So the fullness of its Red channel is INVERSELY porportional to its
     * hueDecimal, the distance from the beginning of this region: 1 - 0.167 = 0.833
     *
   * DgiYlw is hue 60, which puts it at the very beginning of the yellow-into-green region.
     *
     * This means its Red Channel and its Green channel must both be full.
     * Like Citron, the fullness of DgiYlw's Red channel is inversely proporional to its
     * hueDecimal, which is 0. Therefore DgiYlw's Red channel has a fullness of 1.
   *
 * @returns array of values reflecting the spread between channels
 *
 * DgiRed  case 0:  [   R ===== 1       G = x = 0.000   B ===== 0      ]
 *
 * Sunlgt  case 0:  [   R ===== 1       G = x = 0.833   B ===== 0      ]
 *
 * DgiYlw  case 1:  [   R = y = 1.000   G ===== 1       B ===== 0      ]
 *
 * Citron  case 1:  [   R = y = 0.833   G ===== 1       B ===== 0      ]
 *
 * Turqse  case 2:  [   R ===== 0       G ===== 1       B = x = 0.500  ]
 *
 * here we see detailed breakdowns of the function's final output for our running examples.
 */

var hueToRelativeChannels = (function (hue) {
  hue = wrapAround(hue, [0, 360]);
  var hueReduced = hue / 60;
  var hueInteger = Math.floor(hueReduced);
  var hueDecimal = hueReduced - hueInteger;
  var x = hueDecimal;
  var y = 1 - hueDecimal;

  switch (hueInteger) {
    case 0:
      return [1, x, 0];

    case 1:
      return [y, 1, 0];

    case 2:
      return [0, 1, x];

    case 3:
      return [0, y, 1];

    case 4:
      return [x, 0, 1];

    case 5:
      return [1, 0, y];

    default:
      throw new Error('invalid hue served:', hue);
  }
});

var inherentLumFromHue = (function (hue) {
  var _hueToRelativeChannel = hueToRelativeChannels(hue),
      factorR = _hueToRelativeChannel[0],
      factorG = _hueToRelativeChannel[1],
      factorB = _hueToRelativeChannel[2];

  var lumR = CHANNEL_SPECIFIC_LUM.R * factorR;
  var lumG = CHANNEL_SPECIFIC_LUM.G * factorG;
  var lumB = CHANNEL_SPECIFIC_LUM.B * factorB;
  var specificLum = lumR + lumG + lumB;
  return specificLum;
});

var maxSatForHueFromTuner = (function (hue, tuner) {
  var maxSat = 255;

  for (var a = -1, b = 0; b < tuner.length; a++, b++) {
    a = wrapAround(a, [0, tuner.length]); // console.log('||| a =', a, 'b =', b)

    var tuningPointA = tuner[a];
    var tuningPointB = tuner[b];

    if (hue >= wrapAround(tuningPointA.hue, [0, 360]) && hue < tuningPointB.hue) {
      // console.log('||| hue', hue, 'is between', tuningPointA.hue, 'and', tuningPointB.hue)
      var _ = hue; // 70

      _ -= tuningPointA.hue; // 70 - 50 = 20

      _ /= tuningPointB.hue - tuningPointA.hue; // 20 / (120 - 50) = 2/7

      _ *= tuningPointB.sat - tuningPointA.sat; // -128 * 2 / 7 = -256 / 7 ~= -37

      _ += tuningPointA.sat;

      maxSat = _;
    }
  } // console.log('--- maxSat', maxSat)


  return maxSat;
});

var hueFromChannels = (function (_ref) {
  var R = _ref.R,
      G = _ref.G,
      B = _ref.B;
  var hue;
  if (R > G && G >= B) hue = 60 * (0 + (G - B) / (R - B));
  if (G >= R && R > B) hue = 60 * (2 - (R - B) / (G - B));
  if (G > B && B >= R) hue = 60 * (2 + (B - R) / (G - R));
  if (B >= G && G > R) hue = 60 * (4 - (G - R) / (B - R));
  if (B > R && R >= G) hue = 60 * (4 + (R - G) / (B - G));
  if (R >= B && B > G) hue = 60 * (6 - (B - G) / (R - G));
  if (R === G && G === B) hue = 0; // console.log('||| found hue', hue)

  return hue;
});

var satFromChannels = (function (_ref) {
  var R = _ref.R,
      G = _ref.G,
      B = _ref.B;
  var sat = Math.max(R, G, B) - Math.min(R, G, B); // console.log('||| found sat', sat)

  return sat;
});

var lumFromChannels = (function (_ref) {
  var R = _ref.R,
      G = _ref.G,
      B = _ref.B;
  var lum = CHANNEL_SPECIFIC_LUM.R * R / 255 + CHANNEL_SPECIFIC_LUM.G * G / 255 + CHANNEL_SPECIFIC_LUM.B * B / 255; // console.log('||| found lum', lum)

  return lum;
});

var channelsToSpec = (function (_ref) {
  var R = _ref.R,
      G = _ref.G,
      B = _ref.B;
  var hue = hueFromChannels({
    R: R,
    G: G,
    B: B
  });
  var sat = satFromChannels({
    R: R,
    G: G,
    B: B
  });
  var lum = lumFromChannels({
    R: R,
    G: G,
    B: B
  });
  return {
    hue: hue,
    sat: sat,
    lum: lum
  };
});

var hexToSpec = (function (hex) {
  var _hexToChannels = hexToChannels(hex),
      R = _hexToChannels.R,
      G = _hexToChannels.G,
      B = _hexToChannels.B;
  /*
  console.log('+++ input hex', hex)
  console.log('||| R', R)
  console.log('||| G', G)
  console.log('||| B', B)
  */


  var _channelsToSpec = channelsToSpec({
    R: R,
    G: G,
    B: B
  }),
      hue = _channelsToSpec.hue,
      sat = _channelsToSpec.sat,
      lum = _channelsToSpec.lum;

  return {
    hue: hue,
    sat: sat,
    lum: lum
  };
});

var specToChannels = (function (_ref) {
  var hue = _ref.hue,
      sat = _ref.sat,
      lum = _ref.lum,
      _ref$prefer = _ref.prefer,
      prefer = _ref$prefer === void 0 ? 'sat' : _ref$prefer,
      tuner = _ref.tuner;

  /*
  console.log('||| hue', hue)
  console.log('||| sat', sat)
  console.log('||| lum', lum)
  console.log('||| prefer', prefer)
  */
  var relativeChannels = hueToRelativeChannels(hue);

  function makeMinChannels(trueSaturation) {
    var makeMinChannel = function makeMinChannel(idx) {
      return Math.round(relativeChannels[idx] * trueSaturation);
    };

    return {
      R: makeMinChannel(0),
      G: makeMinChannel(1),
      B: makeMinChannel(2)
    };
  }

  var trueSaturation;
  var trueLuminosity;
  var minChannels;
  var minLum = 0;
  var maxLum = 1;
  var maxSat = maxSatForHueFromTuner(hue, tuner);

  if (prefer === 'sat') {
    trueSaturation = Math.min(Math.floor(sat), maxSat);
    minChannels = makeMinChannels(trueSaturation);
    var maxChannels = {
      R: minChannels.R + 255 - trueSaturation,
      G: minChannels.G + 255 - trueSaturation,
      B: minChannels.B + 255 - trueSaturation
    };
    minLum = lumFromChannels(minChannels);
    maxLum = lumFromChannels(maxChannels);
    trueLuminosity = funnel(lum, [minLum, maxLum]);
    /*
    console.log('||| trueSaturation', trueSaturation)
    console.log('||| idealLum', lum)
    console.log('||| minChannels', minChannels)
    console.log('||| minLum', minLum)
    console.log('||| maxChannels', maxChannels)
    console.log('||| maxLum', maxLum)
    console.log('||| trueLuminosity', trueLuminosity)
    */
  }

  if (prefer === 'lum') {
    trueLuminosity = funnel(lum, [0, 1]);
    var specificLum = inherentLumFromHue(hue);
    maxSat = Math.min(maxSat, Math.round(trueLuminosity <= specificLum ? 255 * (trueLuminosity / specificLum) : 255 * (1 - trueLuminosity) / (1 - specificLum)));
    trueSaturation = Math.min(sat, maxSat);
    minChannels = makeMinChannels(trueSaturation);
    minLum = lumFromChannels(minChannels);
    /*
    console.log('||| trueLuminosity', trueLuminosity)
    console.log('||| lumAtSat255', lumAtSat255)
    console.log('||| maxSat', maxSat)
    console.log('||| trueSaturation', trueSaturation)
    console.log('||| minChannels', minChannels)
    */
  }

  var maxWhite = 255 - Math.max.apply(Math, Object.values(minChannels));
  var white = funnel(Math.round((trueLuminosity - minLum) * 255), [0, maxWhite]);
  var _R$G$B = {
    R: minChannels.R + white,
    G: minChannels.G + white,
    B: minChannels.B + white
  },
      R = _R$G$B.R,
      G = _R$G$B.G,
      B = _R$G$B.B;
  /*
  console.log('||| maxWhite', maxWhite)
  console.log('||| white', white)
  console.log('||| R', R)
  console.log('||| G', G)
  console.log('||| B', B)
  */

  return {
    channels: {
      R: R,
      G: G,
      B: B
    },
    fix: {
      sat: trueSaturation,
      lum: trueLuminosity
    },
    limit: {
      sat: [0, maxSat],
      lum: [minLum, maxLum]
    }
  };
});

var channelsToHex = (function (channels) {
  return "#" + Object.values(channels).map(function (channel) {
    var channelHex = channel.toString(16);
    if (channelHex.length === 1) channelHex = 0 + channelHex;
    return channelHex;
  }).join('');
});

var specToHexFixLimit = (function (_ref) {
  var hue = _ref.hue,
      sat = _ref.sat,
      lum = _ref.lum,
      prefer = _ref.prefer,
      tuner = _ref.tuner;

  var _specToChannels = specToChannels({
    hue: hue,
    sat: sat,
    lum: lum,
    prefer: prefer,
    tuner: tuner
  }),
      channels = _specToChannels.channels,
      fix = _specToChannels.fix,
      limit = _specToChannels.limit;

  var R = channels.R,
      G = channels.G,
      B = channels.B;
  var hex = channelsToHex({
    R: R,
    G: G,
    B: B
  }); // console.log('--- newHex', hex)

  return {
    hex: hex,
    fix: fix,
    limit: limit
  };
});

var specToHex = (function (_ref) {
  var hue = _ref.hue,
      sat = _ref.sat,
      lum = _ref.lum,
      prefer = _ref.prefer,
      tuner = _ref.tuner;

  var _specToHexFixLimit = specToHexFixLimit({
    hue: hue,
    sat: sat,
    lum: lum,
    prefer: prefer,
    tuner: tuner
  }),
      hex = _specToHexFixLimit.hex;

  return hex;
});

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function interpolateChanges(rootSpec, currentStep, axes) {
  var changes = {};

  for (var axisIdx = 0; axisIdx < axes.length; axisIdx++) {
    var currentAxis = axes[axisIdx];
    var to = currentAxis.to,
        attribute = currentAxis.attribute;
    var from = typeof currentAxis.from === 'number' ? currentAxis.from : rootSpec[attribute];
    var value = interpolate({
      completionRatio: currentStep,
      range: [to, from]
    });
    changes[attribute] = value;
  } // console.log('>>> changes', changes)


  return changes;
}

function gradientToHexArray(rootSpec, gradient, tuner) {
  var steps = gradient.steps,
      axes = gradient.axes,
      prefer = gradient.prefer;
  var hexArray = [];

  for (var stepIdx = 0; stepIdx < steps.length; stepIdx++) {
    var currentStep = steps[stepIdx];
    var changes = interpolateChanges(rootSpec, currentStep, axes);
    var hex = specToHex(_extends({}, rootSpec, changes, {
      prefer: prefer,
      tuner: tuner
    }));
    hexArray.push(hex);
  }

  return hexArray;
}

var gradientsToHexArrays = (function (originalColorObject, tuner) {
  var gradients = originalColorObject.gradients;
  var hue = originalColorObject.hue,
      sat = originalColorObject.sat,
      lum = originalColorObject.lum;
  var rootSpec = {
    hue: hue,
    sat: sat,
    lum: lum
  };
  var hexArrays = [];

  for (var gradientIdx = 0; gradientIdx < gradients.length; gradientIdx++) {
    var gradient = gradients[gradientIdx];
    var hexGroup = gradientToHexArray(rootSpec, gradient, tuner);
    hexArrays.push(hexGroup);
  }

  return hexArrays;
});

var getStandout = (function (hex) {
  var _hexToSpec = hexToSpec(hex),
      lum = _hexToSpec.lum;

  var standOut = lum > 0.666 ? 'black' : 'white';
  return standOut;
});

var getOffset = (function (_ref) {
  var hex = _ref.hex,
      tuner = _ref.tuner,
      offsets = _ref.offsets;
  var originSpec = hexToSpec(hex);
  var changes = {};

  for (var index = 0; index < offsets.length; index++) {
    var _offsets$index = offsets[index],
        attribute = _offsets$index.attribute,
        value = _offsets$index.value;
    var originValue = originSpec[attribute];
    changes[attribute] = attribute === 'lum' && originValue > -1 * value ? originValue + value : originValue - value * 1.67;
  }

  var newHex = specToHex(_extends({}, originSpec, changes, {
    prefer: 'lum',
    tuner: tuner
  }));
  return newHex;
});

exports.getOffset = getOffset;
exports.getStandout = getStandout;
exports.gradientsToHexArrays = gradientsToHexArrays;
exports.hexToSpec = hexToSpec;
exports.specToHex = specToHex;
exports.specToHexFixLimit = specToHexFixLimit;
//# sourceMappingURL=luum.js.map
