'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var hexToChannels = (hex => {
  const channel = idx => parseInt(hex.substr(idx * 2 + 1, 2), 16);

  return {
    R: channel(0),
    G: channel(1),
    B: channel(2)
  };
});

const CHANNEL_SPECIFIC_LUM = {
  R: 0.35,
  G: 0.5,
  B: 0.15
};
/*
const IM_WONDERING_IF_THIS_IS_NOT_VERY_ACCURATE = {
  R: 0.2126,
  G: 0.7152,
  B: 0.0722,
}
*/

function funnel (value, [min, max]) {
  return value > max ? max : value < min ? min : value;
}

var interpolate = (({
  completionRatio = 0.5,
  range: [to, from]
}) => {
  const value = from + completionRatio * (to - from);
  /*
  console.log('||| to', to, 'from', from)
  console.log("||| completionRatio", completionRatio)
  console.log("||| value", value)
  */

  return value;
});

function wrapAround(value, [min, max]) {
  while (value >= max) value -= max;

  while (value < min) value += max;

  return value;
}

/**
 * Gives us the relative values of the channels,
 * irrespective of the white light beneath them.
 * @param {number} hue - in degrees. Gets safely wrapped around first thing.
 *
 * DgiRed    0 ->   0
 *
 * Sunlgt   50 ->  50
 *
 * DgiYlw   60 ->  60
 *
 * Citron   70 ->  70
 *
 * Turqse  510 -> 150
 *
 * @const {number} hueReduced - hue 0-359.9 is now a floating point 0-5.999.
 *
 * DgiRed    0 ->   0  ~  0.000
 *
 * Sunlgt   50 -> 5/6  ~  0.833
 *
 * DgiYlw   60 ->   1  ~  1.000
 *
 * Citron   70 -> 7/6  ~  1.167
 *
 * Turqse  150 -> 5/2  ~  2.500
 *
 * @const {number} hueInteger - from 1-6. Tells us what color region we are in.
 *
 * DgiRed  0.000 -> 0
 *                : red-into-yellow region
 *
 * Sunlgt  0.833 -> 0
 *                : red-into-yellow region
 *
 * DgiYlw  1.000 -> 1
 *                : yellow-into-green region
 *
 * Citron  1.167 -> 1
 *                : yellow-into-green region
 *
 * Turqse  2.500 -> 2
 *                : green-into-cyan region
 *
 * hueInteger is the 'whole number' piece of hueReduced.
 * FYI, the six color regions are bounded by red, yellow, green, cyan, blue, magenta.
 *
 * @const {number} hueDecimal - tells where we are in this region.
 *
 * DgiRed  0.000 -> 0.000
 *                : at the very beginning
 *
 * Sunlgt  0.833 -> 0.833
 *                : near the end
 *
 * DgiYlw  1.000 -> 0.000
 *                : at the very beginning
 *
 * Citron  1.167 -> 0.167
 *                : near the beginning
 *
 * Turqse  2.500 -> 0.500
 *                : at the halfway point
 *
 * hueDecimal is the 'fraction' piece of hueReduced.
 * we are going to use this number to determine the value of the in-between channel.
 *
 * @const {number} x - used in primary-secondary transitions like Red into Yellow
 * @const {number} y - used in secondary-primary transitions like Yellow into Green
   * To understand the function of x and y, take the difference between
   *
   * Sunlgt (hue 50),
   * Citron (hue 70),
   * DgiYlw (hue 60),
   *
   * as an instructive case. These colors are all basically yellow.
   *
   * Sunlgt is hue 50, which puts it near the end of the red-into-yellow region.
     *
     * This means its Red channel is full, and its Green channel is almost full.
     * The fullness of its Green channel is directly proportional to its
     * hueDecimal, the distance from the beginning of this region: 0.833
     *
   * Citron is hue 70, which puts it near the beginning of the yellow-into-green region.
     *
     * This means its Red channel is ALMOST FULL, and its Green channel is FULL.
     * So the fullness of its Red channel is INVERSELY porportional to its
     * hueDecimal, the distance from the beginning of this region: 1 - 0.167 = 0.833
     *
   * DgiYlw is hue 60, which puts it at the very beginning of the yellow-into-green region.
     *
     * This means its Red Channel and its Green channel must both be full.
     * Like Citron, the fullness of DgiYlw's Red channel is inversely proporional to its
     * hueDecimal, which is 0. Therefore DgiYlw's Red channel has a fullness of 1.
   *
 * @returns array of values reflecting the spread between channels
 *
 * DgiRed  case 0:  [   R ===== 1       G = x = 0.000   B ===== 0      ]
 *
 * Sunlgt  case 0:  [   R ===== 1       G = x = 0.833   B ===== 0      ]
 *
 * DgiYlw  case 1:  [   R = y = 1.000   G ===== 1       B ===== 0      ]
 *
 * Citron  case 1:  [   R = y = 0.833   G ===== 1       B ===== 0      ]
 *
 * Turqse  case 2:  [   R ===== 0       G ===== 1       B = x = 0.500  ]
 *
 * here we see detailed breakdowns of the function's final output for our running examples.
 */

var hueToRelativeChannels = (hue => {
  hue = wrapAround(hue, [0, 360]);
  const hueReduced = hue / 60;
  const hueInteger = Math.floor(hueReduced);
  const hueDecimal = hueReduced - hueInteger;
  const x = hueDecimal;
  const y = 1 - hueDecimal;

  switch (hueInteger) {
    case 0:
      return [1, x, 0];

    case 1:
      return [y, 1, 0];

    case 2:
      return [0, 1, x];

    case 3:
      return [0, y, 1];

    case 4:
      return [x, 0, 1];

    case 5:
      return [1, 0, y];

    default:
      throw new Error('invalid hue served:', hue);
  }
});

var inherentLumFromHue = (hue => {
  const [factorR, factorG, factorB] = hueToRelativeChannels(hue);
  const lumR = CHANNEL_SPECIFIC_LUM.R * factorR;
  const lumG = CHANNEL_SPECIFIC_LUM.G * factorG;
  const lumB = CHANNEL_SPECIFIC_LUM.B * factorB;
  const specificLum = lumR + lumG + lumB;
  return specificLum;
});

var maxSatForHueFromTuner = ((hue, tuner) => {
  let maxSat = 255;

  for (let a = -1, b = 0; b < tuner.length; a++, b++) {
    a = wrapAround(a, [0, tuner.length]); // console.log('||| a =', a, 'b =', b)

    const tuningPointA = tuner[a];
    const tuningPointB = tuner[b];

    if (hue >= wrapAround(tuningPointA.hue, [0, 360]) && hue < tuningPointB.hue) {
      // console.log('||| hue', hue, 'is between', tuningPointA.hue, 'and', tuningPointB.hue)
      let _ = hue; // 70

      _ -= tuningPointA.hue; // 70 - 50 = 20

      _ /= tuningPointB.hue - tuningPointA.hue; // 20 / (120 - 50) = 2/7

      _ *= tuningPointB.sat - tuningPointA.sat; // -128 * 2 / 7 = -256 / 7 ~= -37

      _ += tuningPointA.sat;

      maxSat = _;
    }
  } // console.log('--- maxSat', maxSat)


  return maxSat;
});

var hueFromChannels = (({
  R,
  G,
  B
}) => {
  let hue;
  if (R > G && G >= B) hue = 60 * (0 + (G - B) / (R - B));
  if (G >= R && R > B) hue = 60 * (2 - (R - B) / (G - B));
  if (G > B && B >= R) hue = 60 * (2 + (B - R) / (G - R));
  if (B >= G && G > R) hue = 60 * (4 - (G - R) / (B - R));
  if (B > R && R >= G) hue = 60 * (4 + (R - G) / (B - G));
  if (R >= B && B > G) hue = 60 * (6 - (B - G) / (R - G));
  if (R === G && G === B) hue = 0; // console.log('||| found hue', hue)

  return hue;
});

var satFromChannels = (({
  R,
  G,
  B
}) => {
  const sat = Math.max(R, G, B) - Math.min(R, G, B); // console.log('||| found sat', sat)

  return sat;
});

var lumFromChannels = (({
  R,
  G,
  B
}) => {
  const lum = CHANNEL_SPECIFIC_LUM.R * R / 255 + CHANNEL_SPECIFIC_LUM.G * G / 255 + CHANNEL_SPECIFIC_LUM.B * B / 255; // console.log('||| found lum', lum)

  return lum;
});

var channelsToSpec = (({
  R,
  G,
  B
}) => {
  const hue = hueFromChannels({
    R,
    G,
    B
  });
  const sat = satFromChannels({
    R,
    G,
    B
  });
  const lum = lumFromChannels({
    R,
    G,
    B
  });
  return {
    hue,
    sat,
    lum
  };
});

var hexToSpec = (hex => {
  const {
    R,
    G,
    B
  } = hexToChannels(hex);
  /*
  console.log('+++ input hex', hex)
  console.log('||| R', R)
  console.log('||| G', G)
  console.log('||| B', B)
  */

  const {
    hue,
    sat,
    lum
  } = channelsToSpec({
    R,
    G,
    B
  });
  return {
    hue,
    sat,
    lum
  };
});

var specToChannels = (({
  hue,
  sat,
  lum,
  prefer = 'sat',
  tuner
}) => {
  /*
  console.log('||| hue', hue)
  console.log('||| sat', sat)
  console.log('||| lum', lum)
  console.log('||| prefer', prefer)
  */
  const relativeChannels = hueToRelativeChannels(hue);

  function makeMinChannels(trueSaturation) {
    const makeMinChannel = idx => Math.round(relativeChannels[idx] * trueSaturation);

    return {
      R: makeMinChannel(0),
      G: makeMinChannel(1),
      B: makeMinChannel(2)
    };
  }

  let trueSaturation;
  let trueLuminosity;
  let minChannels;
  let minLum = 0;
  let maxLum = 1;
  let maxSat = maxSatForHueFromTuner(hue, tuner);

  if (prefer === 'sat') {
    trueSaturation = Math.min(Math.floor(sat), maxSat);
    minChannels = makeMinChannels(trueSaturation);
    const maxChannels = {
      R: minChannels.R + 255 - trueSaturation,
      G: minChannels.G + 255 - trueSaturation,
      B: minChannels.B + 255 - trueSaturation
    };
    minLum = lumFromChannels(minChannels);
    maxLum = lumFromChannels(maxChannels);
    trueLuminosity = funnel(lum, [minLum, maxLum]);
    /*
    console.log('||| trueSaturation', trueSaturation)
    console.log('||| idealLum', lum)
    console.log('||| minChannels', minChannels)
    console.log('||| minLum', minLum)
    console.log('||| maxChannels', maxChannels)
    console.log('||| maxLum', maxLum)
    console.log('||| trueLuminosity', trueLuminosity)
    */
  }

  if (prefer === 'lum') {
    trueLuminosity = funnel(lum, [0, 1]);
    const specificLum = inherentLumFromHue(hue);
    maxSat = Math.min(maxSat, Math.round(trueLuminosity <= specificLum ? 255 * (trueLuminosity / specificLum) : 255 * (1 - trueLuminosity) / (1 - specificLum)));
    trueSaturation = Math.min(sat, maxSat);
    minChannels = makeMinChannels(trueSaturation);
    minLum = lumFromChannels(minChannels);
    /*
    console.log('||| trueLuminosity', trueLuminosity)
    console.log('||| lumAtSat255', lumAtSat255)
    console.log('||| maxSat', maxSat)
    console.log('||| trueSaturation', trueSaturation)
    console.log('||| minChannels', minChannels)
    */
  }

  const maxWhite = 255 - Math.max(...Object.values(minChannels));
  const white = funnel(Math.round((trueLuminosity - minLum) * 255), [0, maxWhite]);
  const {
    R,
    G,
    B
  } = {
    R: minChannels.R + white,
    G: minChannels.G + white,
    B: minChannels.B + white
  };
  /*
  console.log('||| maxWhite', maxWhite)
  console.log('||| white', white)
  console.log('||| R', R)
  console.log('||| G', G)
  console.log('||| B', B)
  */

  return {
    channels: {
      R,
      G,
      B
    },
    fix: {
      sat: trueSaturation,
      lum: trueLuminosity
    },
    limit: {
      sat: [0, maxSat],
      lum: [minLum, maxLum]
    }
  };
});

var channelsToHex = (channels => `#${Object.values(channels).map(channel => {
  let channelHex = channel.toString(16);
  if (channelHex.length === 1) channelHex = 0 + channelHex;
  return channelHex;
}).join('')}`);

var specToHexFixLimit = (({
  hue,
  sat,
  lum,
  prefer,
  tuner
}) => {
  const {
    channels,
    fix,
    limit
  } = specToChannels({
    hue,
    sat,
    lum,
    prefer,
    tuner
  });
  const {
    R,
    G,
    B
  } = channels;
  const hex = channelsToHex({
    R,
    G,
    B
  }); // console.log('--- newHex', hex)

  return {
    hex,
    fix,
    limit
  };
});

var specToHex = (({
  hue,
  sat,
  lum,
  prefer,
  tuner
}) => {
  const {
    hex
  } = specToHexFixLimit({
    hue,
    sat,
    lum,
    prefer,
    tuner
  });
  return hex;
});

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function interpolateChanges(rootSpec, currentStep, axes) {
  const changes = {};

  for (let axisIdx = 0; axisIdx < axes.length; axisIdx++) {
    const currentAxis = axes[axisIdx];
    const {
      to,
      attribute
    } = currentAxis;
    const from = typeof currentAxis.from === 'number' ? currentAxis.from : rootSpec[attribute];
    const value = interpolate({
      completionRatio: currentStep,
      range: [to, from]
    });
    changes[attribute] = value;
  } // console.log('>>> changes', changes)


  return changes;
}

function gradientToHexArray(rootSpec, gradient, tuner) {
  const {
    steps,
    axes,
    prefer
  } = gradient;
  const hexArray = [];

  for (let stepIdx = 0; stepIdx < steps.length; stepIdx++) {
    const currentStep = steps[stepIdx];
    const changes = interpolateChanges(rootSpec, currentStep, axes);
    const hex = specToHex(_extends({}, rootSpec, changes, {
      prefer,
      tuner
    }));
    hexArray.push(hex);
  }

  return hexArray;
}

var gradientsToHexArrays = ((originalColorObject, tuner) => {
  const {
    gradients
  } = originalColorObject;
  const {
    hue,
    sat,
    lum
  } = originalColorObject;
  const rootSpec = {
    hue,
    sat,
    lum
  };
  const hexArrays = [];

  for (let gradientIdx = 0; gradientIdx < gradients.length; gradientIdx++) {
    const gradient = gradients[gradientIdx];
    const hexGroup = gradientToHexArray(rootSpec, gradient, tuner);
    hexArrays.push(hexGroup);
  }

  return hexArrays;
});

var getStandout = (hex => {
  const {
    lum
  } = hexToSpec(hex);
  const standOut = lum > 0.666 ? 'black' : 'white';
  return standOut;
});

var getOffset = (({
  hex,
  tuner,
  offsets
}) => {
  const originSpec = hexToSpec(hex);
  const changes = {};

  for (let index = 0; index < offsets.length; index++) {
    const {
      attribute,
      value
    } = offsets[index];
    const originValue = originSpec[attribute];
    changes[attribute] = attribute === 'lum' && originValue > -1 * value ? originValue + value : originValue - value * 1.67;
  }

  const newHex = specToHex(_extends({}, originSpec, changes, {
    prefer: 'lum',
    tuner
  }));
  return newHex;
});

exports.getOffset = getOffset;
exports.getStandout = getStandout;
exports.gradientsToHexArrays = gradientsToHexArrays;
exports.hexToSpec = hexToSpec;
exports.specToHex = specToHex;
exports.specToHexFixLimit = specToHexFixLimit;
//# sourceMappingURL=luum.js.map
